package com.icing.dialer.services

import android.telecom.Connection
import android.telecom.ConnectionService
import android.telecom.PhoneAccountHandle
import android.telecom.TelecomManager
import android.telecom.DisconnectCause
import android.net.Uri
import android.os.Bundle
import android.util.Log
import io.flutter.plugin.common.MethodChannel

class CallConnectionService : ConnectionService() {
    companion object {
        var channel: MethodChannel? = null
    }

    override fun onCreateOutgoingConnection(
        connectionManagerPhoneAccount: PhoneAccountHandle?,
        request: android.telecom.ConnectionRequest
    ): Connection {
        val connection = object : Connection() {
            override fun onStateChanged(state: Int) {
                super.onStateChanged(state)
                Log.d("CallConnectionService", "Connection state changed: $state")
                val stateStr = when (state) {
                    STATE_DIALING -> "dialing"
                    STATE_ACTIVE -> "active"
                    STATE_DISCONNECTED -> "disconnected"
                    else -> "unknown"
                }
                channel?.invokeMethod("callStateChanged", mapOf("state" to stateStr, "phoneNumber" to request.address.toString()))
            }

            override fun onDisconnect() {
                setDisconnected(DisconnectCause(DisconnectCause.LOCAL))
                destroy()
            }
        }
        connection.setAddress(request.address, TelecomManager.PRESENTATION_ALLOWED)
        connection.setInitialized()
        connection.setDialing() // Start in dialing state
        return connection
    }

    override fun onCreateIncomingConnection(
        connectionManagerPhoneAccount: PhoneAccountHandle?,
        request: android.telecom.ConnectionRequest
    ): Connection {
        val connection = object : Connection() {
            override fun onAnswer() {
                setActive()
            }

            override fun onDisconnect() {
                setDisconnected(DisconnectCause(DisconnectCause.LOCAL))
                destroy()
            }
        }
        connection.setAddress(request.address, TelecomManager.PRESENTATION_ALLOWED)
        connection.setRinging()
        return connection
    }
}